Package: pedmut
Title: Mutation Models for Pedigree Likelihood Computations
Version: 0.9.1
Authors@R: c(
    person("Magnus Dehli", "Vigeland", , "m.d.vigeland@medisin.uio.no", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9134-4962")),
    person("Thore", "Egeland", , "thore.egeland@nmbu.no", role = "ctb",
           comment = c(ORCID = "0000-0002-3465-8885"))
  )
Description: A collection of functions for modelling mutations in
    pedigrees with marker data, as used e.g. in likelihood computations
    with microsatellite data. Implemented models include equal,
    proportional and stepwise models, as well as random models for
    experimental work, and custom models allowing the user to apply any
    valid mutation matrix. Allele lumping is done following the
    lumpability criteria of Kemeny and Snell (1976), ISBN:0387901922.
License: GPL-3
URL: https://github.com/magnusdv/pedmut
Depends: R (>= 4.2.0)
Imports: lpSolve
Suggests: testthat
Encoding: UTF-8
Language: en-GB
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-17 11:45:42 UTC; magnu
Author: Magnus Dehli Vigeland [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9134-4962>),
  Thore Egeland [ctb] (ORCID: <https://orcid.org/0000-0002-3465-8885>)
Maintainer: Magnus Dehli Vigeland <m.d.vigeland@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-17 15:10:09 UTC
Built: R 4.5.2; ; 2026-05-17 15:35:22 UTC; unix
