# Generated by roxygen2: do not edit by hand

S3method(as.matrix,mutationMatrix)
S3method(as.matrix,mutationModel)
S3method(print,mutationMatrix)
S3method(print,mutationModel)
S3method(toString,mutationMatrix)
S3method(toString,mutationModel)
export(adjustRate)
export(alwaysLumpable)
export(findStationary)
export(getParams)
export(isBounded)
export(isLumpable)
export(isMutationMatrix)
export(isMutationModel)
export(isReversible)
export(isStationary)
export(lumpMutSpecial)
export(lumpedMatrix)
export(lumpedModel)
export(makeReversible)
export(makeStationary)
export(mutRate)
export(mutationMatrix)
export(mutationModel)
export(sexEqual)
export(stabilize)
export(stepwiseReversible)
export(validateMutationMatrix)
export(validateMutationModel)
importFrom(lpSolve,lp)
importFrom(stats,runif)
